@@ -22,6 +22,7 @@ from PySide6.QtWidgets import (
     QSizePolicy,
     QSplitter,
     QToolBar,
+    QVBoxLayout,
     QWhatsThis,
     QWidget,
 )
@@ -30,12 +31,8 @@ from foundry import (
     auto_save_level_data_path,
     auto_save_m3l_path,
     auto_save_rom_path,
-    discord_link,
-    enemy_compat_link,
-    feature_video_link,
     get_current_version_name,
     get_latest_version_name,
-    github_link,
     icon,
     main_window_flags_path,
     open_url,
@@ -44,6 +41,11 @@ from foundry import (
 from foundry.game.File import ROM
 from foundry.game.gfx.objects.EnemyItem import EnemyObject
 from foundry.game.gfx.objects.LevelObject import LevelObject
+from foundry.game.gfx.Palette import (
+    PaletteGroup,
+    restore_all_palettes,
+    save_all_palette_groups,
+)
 from foundry.game.level.Level import Level, world_and_level_for_level_address
 from foundry.game.level.LevelRef import LevelRef
 from foundry.game.level.WorldMap import WorldMap
@@ -63,7 +65,7 @@ from foundry.gui.ObjectList import ObjectList
 from foundry.gui.ObjectStatusBar import ObjectStatusBar
 from foundry.gui.ObjectToolBar import ObjectToolBar
 from foundry.gui.ObjectViewer import ObjectViewer
-from foundry.gui.PaletteViewer import PaletteViewer
+from foundry.gui.PaletteViewer import PaletteViewer, SidePalette
 from foundry.gui.settings import SETTINGS, save_settings
 from foundry.gui.SettingsDialog import POWERUPS, SettingsDialog
 from foundry.gui.SpinnerPanel import SpinnerPanel
@@ -84,8 +86,15 @@ with open(main_window_flags_path, "r") as data:
     main_window_flags = json.loads(data.read())
 del data
 
-CHECKABLE_MENU_ITEMS = [flag["id"] for flag in main_window_flags["menu"]["view"].values()]
-CHECKABLE_MENU = {flag["id"]: flag for flag in main_window_flags["menu"]["view"].values()}
+menu_options_groups = main_window_flags["menu"]["view"]["options"]
+menu_options = {}
+for menu_option in menu_options_groups:
+    menu_options.update(menu_option)
+
+CHECKABLE_MENU_ITEMS = [option["id"] for option in menu_options.values()]
+CHECKABLE_MENU = {option["id"]: option for option in menu_options.values()}
+del menu_options_groups
+del menu_options
 
 ID_PROP = "ID"
 
@@ -100,102 +109,53 @@ class MainWindow(QMainWindow):
     def __init__(self, path_to_rom=""):
         super(MainWindow, self).__init__()
 
-        self.setWindowIcon(icon("foundry.ico"))
-        self.setStyleSheet(SETTINGS["gui_style"])
+        self.setWindowIcon(icon(main_window_flags["icon"]))
+        self.setStyleSheet(SETTINGS[main_window_flags["style"]])
 
-        file_menu = QMenu("File")
+        for menu_name, menu in main_window_flags["menu"].items():
+            qmenu = QMenu(menu["name"])
+            if menu.get("attribute", False):
+                setattr(self, f"{menu_name}_menu", qmenu)
+            if "action" in menu:
+                qmenu.triggered.connect(getattr(self, menu["action"]))
+            menu_type = menu["type"]
 
-        open_rom_action = file_menu.addAction("&Open ROM")
-        open_rom_action.triggered.connect(self.on_open_rom)
-        self.open_m3l_action = file_menu.addAction("&Open M3L")
-        self.open_m3l_action.triggered.connect(self.on_open_m3l)
+            for index, option_group in enumerate(menu["options"]):
+                if index != 0:
+                    qmenu.addSeparator()
+                if menu_type == "actions":
+                    for base_name, option in option_group.items():
+                        name: str = option["name"]
+                        action = qmenu.addAction(name)
+                        if option.get("attribute", False):
+                            setattr(self, f"{base_name}_action", action)
+                        if "action" in option:
+                            method = getattr(self, option["action"])
+                        elif "link" in option:
 
-        file_menu.addSeparator()
+                            def call_link(url: str):
+                                def call_link():
+                                    return open_url(url)
 
-        self.save_rom_action = file_menu.addAction("&Save ROM")
-        self.save_rom_action.triggered.connect(self.on_save_rom)
-        self.save_rom_as_action = file_menu.addAction("&Save ROM as ...")
-        self.save_rom_as_action.triggered.connect(self.on_save_rom_as)
-        self.save_m3l_action = file_menu.addAction("&Save M3L")
-        self.save_m3l_action.triggered.connect(self.on_save_m3l)
+                                return call_link
 
-        file_menu.addSeparator()
-        settings_action = file_menu.addAction("&Settings")
-        settings_action.triggered.connect(self._on_show_settings)
-        file_menu.addSeparator()
-        exit_action = file_menu.addAction("&Exit")
-        exit_action.triggered.connect(lambda _: self.close())
+                            method = call_link(option["link"])
+                        else:
+                            raise NotImplementedError
+                        if option.get("wrapped", False):
+                            action.triggered.connect(lambda *_: method())
+                        else:
+                            action.triggered.connect(method)
 
-        self.menuBar().addMenu(file_menu)
+                elif menu_type == "settings":
+                    for option in option_group.values():
+                        name: str = option["display_name"]
+                        action = qmenu.addAction(name)
+                        action.setProperty(ID_PROP, option["id"])
+                        action.setCheckable(True)
+                        action.setChecked(SETTINGS[option["name"]])
 
-        self.level_menu = QMenu("Level")
-
-        self.select_level_action = self.level_menu.addAction("&Select Level")
-        self.select_level_action.triggered.connect(self.open_level_selector)
-
-        self.reload_action = self.level_menu.addAction("&Reload Level")
-        self.reload_action.triggered.connect(self.reload_level)
-        self.level_menu.addSeparator()
-        self.edit_header_action = self.level_menu.addAction("&Edit Header")
-        self.edit_header_action.triggered.connect(self.on_header_editor)
-        self.edit_autoscroll = self.level_menu.addAction("Edit Autoscrolling")
-        self.edit_autoscroll.triggered.connect(self.on_edit_autoscroll)
-
-        self.menuBar().addMenu(self.level_menu)
-
-        self.object_menu = QMenu("Objects")
-
-        view_blocks_action = self.object_menu.addAction("&View Blocks")
-        view_blocks_action.triggered.connect(self.on_block_viewer)
-        view_objects_action = self.object_menu.addAction("&View Objects")
-        view_objects_action.triggered.connect(self.on_object_viewer)
-        self.object_menu.addSeparator()
-        view_palettes_action = self.object_menu.addAction("View Object Palettes")
-        view_palettes_action.triggered.connect(self.on_palette_viewer)
-
-        self.menuBar().addMenu(self.object_menu)
-
-        self.view_menu = QMenu("View")
-        self.view_menu.triggered.connect(self.on_menu)
-
-        for flag in CHECKABLE_MENU.values():
-            action: QAction = self.view_menu.addAction(flag["display_name"])
-            action.setProperty(ID_PROP, flag["id"])
-            action.setCheckable(True)
-            action.setChecked(SETTINGS[flag["name"]])
-            if flag.get("separator", False):
-                self.view_menu.addSeparator()
-
-        self.view_menu.addSeparator()
-        self.view_menu.addAction("&Save Screenshot of Level").triggered.connect(self.on_screenshot)
-
-        self.menuBar().addMenu(self.view_menu)
-
-        help_menu = QMenu("Help")
-
-        update_action = help_menu.addAction("Check for updates")
-        update_action.triggered.connect(self.on_check_for_update)
-
-        help_menu.addSeparator()
-
-        video_action = help_menu.addAction("Feature Video on YouTube")
-        video_action.triggered.connect(lambda: open_url(feature_video_link))
-
-        github_action = help_menu.addAction("Github Repository")
-        github_action.triggered.connect(lambda: open_url(github_link))
-
-        discord_action = help_menu.addAction("SMB3 Rom Hacking Discord")
-        discord_action.triggered.connect(lambda: open_url(discord_link))
-
-        help_menu.addSeparator()
-
-        enemy_compat_action = help_menu.addAction("Enemy Compatibility")
-        enemy_compat_action.triggered.connect(lambda: open_url(enemy_compat_link))
-
-        about_action = help_menu.addAction("&About")
-        about_action.triggered.connect(self.on_about)
-
-        self.menuBar().addMenu(help_menu)
+            self.menuBar().addMenu(qmenu)
 
         self.block_viewer = None
         self.object_viewer = None
@@ -227,6 +187,17 @@ class MainWindow(QMainWindow):
         self.level_size_bar = LevelSizeBar(self, self.level_ref)
         self.enemy_size_bar = EnemySizeBar(self, self.level_ref)
 
+        size_and_palette = QWidget()
+        size_and_palette.setLayout(QHBoxLayout())
+        size_and_palette.layout().setContentsMargins(0, 0, 0, 0)
+
+        size_layout = QVBoxLayout()
+        size_layout.addWidget(self.level_size_bar)
+        size_layout.addWidget(self.enemy_size_bar)
+
+        size_and_palette.layout().addLayout(size_layout, stretch=1)
+        size_and_palette.layout().addWidget(SidePalette(self.level_ref))
+
         self.jump_list = JumpList(self, self.level_ref)
         self.jump_list.add_jump.connect(self.on_jump_added)
         self.jump_list.edit_jump.connect(self.on_jump_edit)
@@ -263,6 +234,7 @@ class MainWindow(QMainWindow):
 
         level_toolbar.addWidget(self.spinner_panel)
         level_toolbar.addWidget(self.object_dropdown)
+        level_toolbar.addWidget(size_and_palette)
         level_toolbar.addWidget(self.level_size_bar)
         level_toolbar.addWidget(self.enemy_size_bar)
         level_toolbar.addWidget(splitter)
@@ -292,6 +264,8 @@ class MainWindow(QMainWindow):
         self.menu_toolbar.addSeparator()
         self.menu_toolbar.addAction(icon("folder.svg"), "Open ROM").triggered.connect(self.on_open_rom)
         self.menu_toolbar.addAction(icon("save.svg"), "Save Level").triggered.connect(self.on_save_rom)
+        self.menu_toolbar_save_action = self.menu_toolbar.addAction(icon("save.svg"), "Save Level")
+        self.menu_toolbar_save_action.triggered.connect(self.on_save_rom)
         self.menu_toolbar.addSeparator()
 
         self.undo_action = self.menu_toolbar.addAction(icon("rotate-ccw.svg"), "Undo Action")
@@ -374,6 +348,11 @@ class MainWindow(QMainWindow):
 
         self.jump_destination_action.setEnabled(self.level_ref.level.has_next_area)
 
+        level_has_changed = self.level_ref.level.changed and not self.level_ref.level.undo_stack.is_empty
+        level_is_m3l = not self.level_ref.level.attached_to_rom
+
+        self.menu_toolbar_save_action.setEnabled(level_has_changed or level_is_m3l or PaletteGroup.changed)
+
         self._save_auto_data()
 
     def _on_show_settings(self):
@@ -463,7 +442,9 @@ class MainWindow(QMainWindow):
 
         path_to_temp_rom = temp_dir / "instaplay.rom"
 
-        ROM().save_to(path_to_temp_rom)
+        ROM().save_to(str(path_to_temp_rom))
+
+        temp_rom = self._open_rom(path_to_temp_rom)
 
         if not self._put_current_level_to_level_1_1(path_to_temp_rom):
             return
@@ -471,6 +452,10 @@ class MainWindow(QMainWindow):
         if not self._set_default_powerup(path_to_temp_rom):
             return
 
+        save_all_palette_groups(temp_rom)
+
+        temp_rom.save_to(str(path_to_temp_rom))
+
         arguments = SETTINGS["instaplay_arguments"].replace("%f", str(path_to_temp_rom))
         arguments = shlex.split(arguments, posix=False)
 
@@ -607,7 +592,6 @@ class MainWindow(QMainWindow):
         # set as Raccoon Mario instead of P-wing
         Map_Power_DispResetLocation = 0x3C5A2
         rom.write(Map_Power_DispResetLocation, bytes([nop, nop, nop]))
-        rom.save_to(path_to_rom)
         return True
 
     def on_screenshot(self, _) -> bool:
@@ -713,6 +697,36 @@ class MainWindow(QMainWindow):
         else:
             return True
 
+    def _ask_for_palette_save(self) -> bool:
+        """
+        If Object Palettes have been changed, this function opens a dialog box, asking the user, if they want to save
+        the changes, dismiss them, or cancel whatever they have been doing (probably saving/selecting another level).
+        Saving or restoring Object Palettes is done inside the function if necessary.
+        :return: False, if Cancel was chosen. True, if Palettes were restored or saved to ROM.
+        """
+        if not PaletteGroup.changed:
+            return True
+
+        answer = QMessageBox.question(
+            self,
+            "Please confirm",
+            "You changed some object palettes. This is a change, that potentially affects other levels in this ROM. Do "
+            "you want to save these changes?",
+            QMessageBox.Cancel | QMessageBox.RestoreDefaults | QMessageBox.Yes,
+            QMessageBox.Cancel,
+        )
+
+        if answer == QMessageBox.Cancel:
+            return False
+
+        if answer == QMessageBox.Yes:
+            save_all_palette_groups()
+        elif answer == QMessageBox.RestoreDefaults:
+            restore_all_palettes()
+            self.level_ref.level.reload()
+
+        return True
+
     def on_save_rom(self, _):
         self.save_rom(False)
 
@@ -1051,6 +1065,8 @@ class MainWindow(QMainWindow):
         self.update_gui_for_level()
 
     def update_gui_for_level(self):
+        restore_all_palettes()
+
         self._enable_disable_gui_elements()
 
         self.update_title()
